<?php

namespace SupNext\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Teknisa\Libs\Util\ReportDecorator;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\ErrorHandler\ErrorException;

class ReportConfigController {
  private $connection;
  private $environment;
  private $util;
  private $service;
  private $crudService;

  public function __construct($connection, $environment, $util, $service, $crudService) {
    $this->connection = $connection;
    $this->environment = $environment;
    $this->util = $util;
    $this->service = $service;
    $this->crudService = $crudService;
  }

  public function addReport(DTO\Request\Row $request, DTO\Response $response) {
    try{
      $data = $request->getParameters();
      
      foreach($data as $dt) {
        $exist = $this->verifyReport($dt);

        if(!empty($exist)) {
          return $response->setError(new Error("Foram selecionados relatórios ja cadastrados", 400));
        }
      }

      foreach($data as $dt) {
        $this->service->insertReportConfig($dt);
      }
      $response->addDataSet(new DataSet("addReport", ["message" => "Inclusão realizada com sucesso.", "status_code" => 200]));
    }catch (\Exception $exception) {
      throw new \Exception("Erro ao Incluir: " . $exception->getMessage(), 500);
    }
  }

  public function deleteReport(DTO\Request\Row $request, DTO\Response $response) {
    try{
      $data = $request->getParameters();
      foreach($data as $dt) {
        $this->service->deleteReportConfig($dt);
      }
      $response->addDataSet(new DataSet("deleteReport", ["message" => "Exclusão do relatório realizada com sucesso.", "status_code" => 200]));
    }catch (\Exception $exception) {
      throw new \Exception("Erro ao excluir o relatório: " . $exception->getMessage(), 500);
    }
  }

  private function verifyReport(array $data): array {
    $params = $this->util->createConditions([
      "REPORTNAME"  => $data["REPORTNAME"],
      "REPORTCODE"  => $data["REPORTCODE"],
      "NRORG"       => $this->environment->getNrorg(),
    ]);

    $exist = $this->crudService->getFilteredDataSet("verifyReport", $params)->getRows();
    return $exist;
  }
}