<?php

namespace SupNext\Services\QuoteSheetGrid;

use Doctrine\DBAL\Connection;
use SupNext\Util\Util;

use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;

class AddProductService {
  private $connection;
  private $util;

  public function __construct(Connection $connection, Util $util) {
    $this->connection = $connection;
    $this->util       = $util;
  }

  public function insertItemColeWithIndex(array $data): void {
    try {
      $query = "INSERT INTO ITEMCOLE
                (
                  CDFILIAL,
                  NRCOLEPRECO,
                  CDPRODUTO,
                  CDMARCA,
                  CDFORNECED,
                  CDREGIAO,
                  DTINIVALITM,
                  DTFINVALITM,
                  VRPRECUNIITM,
                  VRINDICEITMCOL,
                  VRPERDESITM,
                  IDTIPFREITM,
                  IDITMCOTPREC,
                  IDATUREGIINC,
                  CDOPERINCL,
                  DTINCLITMCOL,
                  IDATUFORNINC,
                  NRSEQPRELIST,
                  IDITMLISPREC,
                  IDPRECALT,
                  IDITVALDIFLISTA,
                  NRORG
                )
                SELECT :CDFILIAL, :NRCOLEPRECO, :CDPRODUTO, :CDMARCA,
                  :CDFORNECED, :CDREGIAO, COLETAPR.DTINIVALCOL,
                  COLETAPR.DTFINVALCOL, :VRPRECUNIITM, :VRINDICEITMCOL,
                  DECODE(FORNECEDOR.IDTPDESCFORN, 'C',
                  FORNECEDOR.VRPEDESCFORN, 0), FORNECEDOR.IDTPFRETFORN,
                  'S', :P_IDATUREG, :P_CDOPERINCL, SYSDATE,
                  :P_IDATUFORN, :P_NRSEQPRELIST,
                  DECODE(:P_NRSEQPRELIST, NULL, 'N', 'S'),
                  :P_IDPRECALT, :P_IDITVALDIFLISTA, :P_NRORG
                FROM COLETAPR, FORNECEDOR
                WHERE COLETAPR.CDFILIAL = :CDFILIAL
                  AND COLETAPR.NRCOLEPRECO = :NRCOLEPRECO
                  AND FORNECEDOR.CDFORNECED = :CDFORNECED
                  AND COLETAPR.NRORG = FORNECEDOR.NRORG
                  AND FORNECEDOR.NRORG = :P_NRORG
                  AND NOT EXISTS (SELECT CDFORNECED, CDREGIAO
                FROM REFOREG
                WHERE REFOREG.CDFORNECED = :CDFORNECED
                  AND REFOREG.CDREGIAO = :CDREGIAO)
                  AND NOT EXISTS (SELECT CDFILIAL
                FROM ITEMCOLE
                WHERE ITEMCOLE.CDFILIAL = :CDFILIAL
                  AND ITEMCOLE.NRCOLEPRECO = :NRCOLEPRECO
                  AND ITEMCOLE.CDREGIAO = :CDREGIAO
                  AND ITEMCOLE.CDPRODUTO = :CDPRODUTO
                  AND ITEMCOLE.CDMARCA = :CDMARCA
                  AND ITEMCOLE.CDFORNECED = :CDFORNECED)";

      $param = [
        "CDFILIAL"          => $data["CDFILIAL"],
        "NRCOLEPRECO"       => $data["NRCOLEPRECO"],
        "CDPRODUTO"         => $data["CDPRODUTO"],
        "CDMARCA"           => $data["CDMARCA"],
        "CDFORNECED"        => $data["CDFORNECED"],
        "CDREGIAO"          => $data["CDREGIAO"],
        "VRPRECUNIITM"      => $data["VRPRECUNIITM"],
        "VRINDICEITMCOL"    => $data["VRINDICEITMCOL"],
        "P_IDATUREG"        => $data["IDATUREG"],
        "P_CDOPERINCL"      => $data["CDOPERINCL"],
        "P_IDATUFORN"       => $data["IDATUFORN"],
        "P_NRSEQPRELIST"    => $data["NRSEQPRELIST"],
        "P_IDPRECALT"       => $data["IDPRECALT"],
        "P_IDITVALDIFLISTA" => $data["IDITVALDIFLISTA"],
        "P_NRORG"           => $data["NRORG"]
      ];

      $this->util->crud($query, $param);
    } catch (Exception $exception) {
      throw new \Exception("Erro ao inserir produto na cotação: " . $exception->getMessage(), 500);
    }
  }

  public function insertItemColeWithoutIndex(array $data): void {
    try {
      $query = "INSERT INTO ITEMCOLE
                      (
                          CDFILIAL,
                          NRCOLEPRECO,
                          CDPRODUTO,
                          CDMARCA,
                          CDFORNECED,
                          CDREGIAO,
                          DTINIVALITM,
                          DTFINVALITM,
                          VRPRECUNIITM,
                          VRPERDESITM,
                          IDTIPFREITM,
                          IDITMCOTPREC,
                          IDATUREGIINC,
                          CDOPERINCL,
                          DTINCLITMCOL,
                          IDATUFORNINC,
                          NRSEQPRELIST,
                          IDITMLISPREC,
                          IDPRECALT,
                          IDITVALDIFLISTA,
                          NRORG
                        )
                        SELECT :P_CDFILIAL, :P_NRCOLEPRECO, :P_CDPRODUTO, :P_CDMARCA,
                                :P_CDFORNECED, :P_CDREGIAO, COLETAPR.DTINIVALCOL,
                                COLETAPR.DTFINVALCOL, :P_VRPRECUNIITM,
                                DECODE(FORNECEDOR.IDTPDESCFORN, 'C',
                                FORNECEDOR.VRPEDESCFORN, 0), FORNECEDOR.IDTPFRETFORN,
                                'S', :P_IDATUREG, :P_CDOPERINCL, SYSDATE,
                                :P_IDATUFORN, :P_NRSEQPRELIST,
                                DECODE(:P_NRSEQPRELIST, NULL, 'N', 'S'),
                                :P_IDPRECALT, :P_IDITVALDIFLISTA, :P_NRORG
                        FROM COLETAPR, FORNECEDOR
                        WHERE COLETAPR.CDFILIAL = :P_CDFILIAL
                              AND COLETAPR.NRCOLEPRECO = :P_NRCOLEPRECO
                              AND FORNECEDOR.CDFORNECED = :P_CDFORNECED
                              AND COLETAPR.NRORG = FORNECEDOR.NRORG
                              AND FORNECEDOR.NRORG = :P_NRORG
                              AND NOT EXISTS (SELECT CDFORNECED, CDREGIAO
                                              FROM REFOREG
                                              WHERE REFOREG.CDFORNECED = :P_CDFORNECED
                                                    AND REFOREG.CDREGIAO = :P_CDREGIAO)
                                                    AND NOT EXISTS (SELECT CDFILIAL
                                                                    FROM ITEMCOLE
                                                                    WHERE ITEMCOLE.CDFILIAL = :P_CDFILIAL
                                                                          AND ITEMCOLE.NRCOLEPRECO = :P_NRCOLEPRECO
                                                                          AND ITEMCOLE.CDREGIAO = :P_CDREGIAO
                                                                          AND ITEMCOLE.CDPRODUTO = :P_CDPRODUTO
                                                                          AND ITEMCOLE.CDMARCA = :P_CDMARCA
                                                                          AND ITEMCOLE.CDFORNECED = :P_CDFORNECED)";
      $param = [
        "P_CDFILIAL"          => $data["CDFILIAL"],
        "P_NRCOLEPRECO"       => $data["NRCOLEPRECO"],
        "P_CDPRODUTO"         => $data["CDPRODUTO"],
        "P_CDMARCA"           => $data["CDMARCA"],
        "P_CDFORNECED"        => $data["CDFORNECED"],
        "P_CDREGIAO"          => $data["CDREGIAO"],
        "P_VRPRECUNIITM"      => $data["VRPRECUNIITM"],
        "P_IDATUREG"          => $data["IDATUREG"],
        "P_CDOPERINCL"        => $data["CDOPERINCL"],
        "P_IDATUFORN"         => $data["IDATUFORN"],
        "P_NRSEQPRELIST"      => $data["NRSEQPRELIST"],
        "P_IDPRECALT"         => $data["IDPRECALT"],
        "P_IDITVALDIFLISTA"   => $data["IDITVALDIFLISTA"],
        "P_NRORG"             => $data["NRORG"]
      ];

      $this->util->crud($query, $param);
    } catch (\Exception $exception) {
      throw new \Exception("Erro ao inserir produto na cotação: " . $exception->getMessage(), 500);
    }
  }

  public function procPrecosToRegiao(array $data): void {
    try {
      $procedure = new StoredProcedure($this->connection, "PRECOS_TO_REGIAO");

      $procedure->addParam(new Param("P_CDFILIAL",     Param::PARAM_INPUT, $data["CDFILIAL"],     Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_CDFILIPROC",   Param::PARAM_INPUT, $data["CDFILIAL"],     Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_NRCOLEPRECO",  Param::PARAM_INPUT, $data["NRCOLEPRECO"],  Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_CDREGIAO",     Param::PARAM_INPUT, $data["CDREGIAO"],     Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_CDFORNECED",   Param::PARAM_INPUT, $data["CDFORNECED"],   Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_TFORN",        Param::PARAM_INPUT, "",                      Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_CDPRODUTO",    Param::PARAM_INPUT, $data["CDPRODUTO"],    Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_CDMARCA",      Param::PARAM_INPUT, $data["CDMARCA"],      Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_IDATUREG",     Param::PARAM_INPUT, $data["P_IDATUREG"],   Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_IDATUFORN",    Param::PARAM_INPUT, $data["P_IDATUFORN"],  Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_NEW_CDREGIAO", Param::PARAM_INPUT, $data["NEW_CDREGIAO"], Param::PARAM_TYPE_STR));

      $procedure->execute();
    } catch (\Exception $exception) {
      throw new \Exception("Erro ao executar procedure preços para região: " . $exception->getMessage(), 500);
    }
  }

  public function procPrecosToForn(array $data): void {
    try {
      $procedure = new StoredProcedure($this->connection, "PRECOS_TO_FORN");

      $procedure->addParam(new Param("P_CDFILIAL",     Param::PARAM_INPUT, $data["CDFILIAL"],     Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_CDFILIPROC",   Param::PARAM_INPUT, $data["CDFILIAL"],     Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_NRCOLEPRECO",  Param::PARAM_INPUT, $data["NRCOLEPRECO"],  Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_CDREGIAO",     Param::PARAM_INPUT, $data["CDREGIAO"],     Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_CDFORNECED",   Param::PARAM_INPUT, $data["CDFORNECED"],   Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_CDPRODUTO",    Param::PARAM_INPUT, $data["CDPRODUTO"],    Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_CDMARCA",      Param::PARAM_INPUT, $data["CDMARCA"],      Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_IDATUREG",     Param::PARAM_INPUT, $data["P_IDATUREG"],   Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_IDATUFORN",    Param::PARAM_INPUT, $data["P_IDATUFORN"],  Param::PARAM_TYPE_STR));

      $procedure->execute();
    } catch (\Exception $exception) {
      throw new \Exception("Erro ao executar procedure preços para fornecedor: " . $exception->getMessage(), 500);
    }
  }

  public function updateQuatationParameters(array $data): void {
    try {
      $query = "UPDATE COLETAPR
                SET IDTIPCOLREG = 'G'
                WHERE CDFILIAL = :P_CDFILIAL
                      AND NRCOLEPRECO = :P_NRCOLEPRECO
                      AND NVL(IDTIPCOLREG, 'G') NOT IN ('E', 'W')";
      $params = [
          "P_CDFILIAL"      => $data["CDFILIAL"],
          "P_NRCOLEPRECO"   => $data["NRCOLEPRECO"],
      ];

      $this->util->crud($query, $params);
    } catch (\Exception $exception) {
        throw new \Exception("Erro ao atualizar parâmetros cotação: " . $exception->getMessage(), 500);
    }
  }
}