<?php

namespace SupNext\Services\QuoteSheetGrid;

use Doctrine\DBAL\Connection;
use SupNext\Util\Util;

use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;

class AddRegionService {
  private $connection;
  private $util;

  public function __construct(Connection $connection, Util $util) {
    $this->connection = $connection;
    $this->util       = $util;
  }

  public function insertItemColeWithIndex(array $data): void {
    try {
      $query = "INSERT INTO ITEMCOLE
                  (
                    CDFILIAL,
                    NRCOLEPRECO,
                    CDPRODUTO,
                    CDMARCA,
                    CDFORNECED,
                    CDREGIAO,
                    DTINIVALITM,
                    DTFINVALITM,
                    VRPRECUNIITM,
                    VRINDICEITMCOL,
                    VRPERDESITM,
                    IDTIPFREITM,
                    IDITMCOTPREC,
                    IDATUREGIINC,
                    CDOPERINCL,
                    DTINCLITMCOL,
                    IDATUFORNINC,
                    NRSEQPRELIST,
                    IDITMLISPREC,
                    IDPRECALT,
                    IDITVALDIFLISTA,
                    NRORG
                    )
                    SELECT :CDFILIAL, :NRCOLEPRECO, :CDPRODUTO, :CDMARCA,
                     :CDFORNECED, :CDREGIAO, COLETAPR.DTINIVALCOL,
                     COLETAPR.DTFINVALCOL, :VRPRECUNIITM, :VRINDICEITMCOL,
                     DECODE(FORNECEDOR.IDTPDESCFORN, 'C',
                     FORNECEDOR.VRPEDESCFORN, 0), FORNECEDOR.IDTPFRETFORN,
                     'S', :P_IDATUREG, :P_CDOPERINCL, SYSDATE,
                     :P_IDATUFORN, :P_NRSEQPRELIST, DECODE(:P_NRSEQPRELIST,
                     NULL, 'N', 'S'), :P_IDPRECALT, :P_IDITVALDIFLISTA,
                     :P_NRORG
              FROM COLETAPR, FORNECEDOR
              WHERE COLETAPR.CDFILIAL = :CDFILIAL
                AND COLETAPR.NRCOLEPRECO = :NRCOLEPRECO
                AND FORNECEDOR.CDFORNECED = :CDFORNECED
                AND COLETAPR.NRORG = FORNECEDOR.NRORG
                AND FORNECEDOR.NRORG = :P_NRORG
                AND NOT EXISTS (SELECT CDFORNECED, CDREGIAO
              FROM REFOREG
              WHERE REFOREG.CDFORNECED = :CDFORNECED
                  AND REFOREG.CDREGIAO = :CDREGIAO)
                  AND NOT EXISTS (SELECT CDFILIAL
              FROM ITEMCOLE
              WHERE ITEMCOLE.CDFILIAL = :CDFILIAL
                  AND ITEMCOLE.NRCOLEPRECO = :NRCOLEPRECO
                  AND ITEMCOLE.CDREGIAO = :CDREGIAO
                  AND ITEMCOLE.CDPRODUTO = :CDPRODUTO
                  AND ITEMCOLE.CDMARCA = :CDMARCA
                  AND ITEMCOLE.CDFORNECED = :CDFORNECED)";

      $parameters = [
        "CDFILIAL"          => $data["CDFILIAL"],
        "NRCOLEPRECO"       => $data["NRCOLEPRECO"],
        "CDPRODUTO"         => $data["CDPRODUTO"],
        "CDMARCA"           => $data["CDMARCA"],
        "CDFORNECED"        => $data["CDFORNECED"],
        "CDREGIAO"          => $data["CDREGIAO"],
        "VRPRECUNIITM"      => $data["VRPRECUNIITM"],
        "VRINDICEITMCOL"    => $data["VRINDICEITMCOL"],
        "P_IDATUREG"        => $data["IDATUREG"],
        "P_CDOPERINCL"      => $data["CDOPERINCL"],
        "P_IDATUFORN"       => $data["IDATUFORN"],
        "P_NRSEQPRELIST"    => $data["NRSEQPRELIST"],
        "P_IDPRECALT"       => $data["IDPRECALT"],
        "P_IDITVALDIFLISTA" => $data["IDITVALDIFLISTA"],
        "P_NRORG"           => $data["NRORG"]
      ];

      $this->util->crud($query, $parameters);
    } catch (\Exception $exception) {
      throw new \Exception("Error ao inserir cotação: " . $exception->getMessage(), 500);
    }
  }

  public function insertItemColeWithoutIndex(array $data): void {
    try {
      $query = "INSERT INTO ITEMCOLE
                  (
                      CDFILIAL,
                      NRCOLEPRECO,
                      CDPRODUTO,
                      CDMARCA,
                      CDFORNECED,
                      CDREGIAO,
                      DTINIVALITM,
                      DTFINVALITM,
                      VRPRECUNIITM,
                      VRPERDESITM,
                      IDTIPFREITM,
                      IDITMCOTPREC,
                      IDATUREGIINC,
                      CDOPERINCL,
                      DTINCLITMCOL,
                      IDATUFORNINC,
                      NRSEQPRELIST,
                      IDITMLISPREC,
                      IDPRECALT,
                      IDITVALDIFLISTA,
                      NRORG
                   )
                   SELECT :CDFILIAL, :NRCOLEPRECO, :CDPRODUTO, :CDMARCA,
                      :CDFORNECED, :CDREGIAO, COLETAPR.DTINIVALCOL,
                      COLETAPR.DTFINVALCOL, :VRPRECUNIITM,
                      DECODE(FORNECEDOR.IDTPDESCFORN, 'C',
                      FORNECEDOR.VRPEDESCFORN, 0), FORNECEDOR.IDTPFRETFORN,
                      'S', :P_IDATUREG, :P_CDOPERINCL, SYSDATE,
                      :P_IDATUFORN, :P_NRSEQPRELIST, DECODE(:P_NRSEQPRELIST,
                      NULL, 'N', 'S'), :P_IDPRECALT, :P_IDITVALDIFLISTA,
                      :P_NRORG
                  FROM COLETAPR, FORNECEDOR
                  WHERE COLETAPR.CDFILIAL = :CDFILIAL
                      AND COLETAPR.NRCOLEPRECO = :NRCOLEPRECO
                      AND FORNECEDOR.CDFORNECED = :CDFORNECED
                      AND COLETAPR.NRORG = FORNECEDOR.NRORG
                      AND FORNECEDOR.NRORG = :P_NRORG
                      AND NOT EXISTS (SELECT CDFORNECED, CDREGIAO
                  FROM REFOREG
                  WHERE REFOREG.CDFORNECED = :CDFORNECED
                      AND REFOREG.CDREGIAO = :CDREGIAO)
                      AND NOT EXISTS (SELECT CDFILIAL
                  FROM ITEMCOLE
                  WHERE ITEMCOLE.CDFILIAL = :CDFILIAL
                      AND ITEMCOLE.NRCOLEPRECO = :NRCOLEPRECO
                      AND ITEMCOLE.CDREGIAO = :CDREGIAO
                      AND ITEMCOLE.CDPRODUTO = :CDPRODUTO
                      AND ITEMCOLE.CDMARCA = :CDMARCA
                      AND ITEMCOLE.CDFORNECED = :CDFORNECED)";

      $parameters = [
        "CDFILIAL"          => $data["CDFILIAL"],
        "NRCOLEPRECO"       => $data["NRCOLEPRECO"],
        "CDPRODUTO"         => $data["CDPRODUTO"],
        "CDMARCA"           => $data["CDMARCA"],
        "CDFORNECED"        => $data["CDFORNECED"],
        "CDREGIAO"          => $data["CDREGIAO"],
        "VRPRECUNIITM"      => $data["VRPRECUNIITM"],
        "P_IDATUREG"        => $data["IDATUREG"],
        "P_CDOPERINCL"      => $data["CDOPERINCL"],
        "P_IDATUFORN"       => $data["IDATUFORN"],
        "P_NRSEQPRELIST"    => $data["NRSEQPRELIST"],
        "P_IDPRECALT"       => $data["IDPRECALT"],
        "P_IDITVALDIFLISTA" => $data["IDITVALDIFLISTA"],
        "P_NRORG"           => $data["NRORG"]
      ];

      $this->util->crud($query, $parameters);
    } catch (\Exception $exception) {
      throw new \Exception("Erro ao inserir região: " . $exception->getMessage(), 500);
    }
  }

  public function procImpostoItemcol(array $data): void {
    try {
      $procedure = new StoredProcedure($this->connection, "IMPOSTO_ITEMCOL");

      $procedure->addParam(new Param("P_EMPRESA",    Param::PARAM_INPUT, $data["CDEMPRESA"],   Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_INSCESTA",   Param::PARAM_INPUT, $data["CDINSCESTA"],  Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_FORNECEDOR", Param::PARAM_INPUT, $data["CDFORNECED"],  Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_FILCENT",    Param::PARAM_INPUT, $data["CDFILCENT"],   Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_LANCCOLP",   Param::PARAM_INPUT, $data["NRCOLEPRECO"], Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_CDFORNECED", Param::PARAM_INPUT, $data["CDFORNECED"],  Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_CDREGIAO",   Param::PARAM_INPUT, $data["CDREGIAO"],    Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_CDPRODUTO",  Param::PARAM_INPUT, $data["CDPRODUTO"],   Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_CDMARCA",    Param::PARAM_INPUT, $data["CDMARCA"],     Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_INIVALID",   Param::PARAM_INPUT, $data["DTCOLEPREC"],  Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_TRANSACAO",  Param::PARAM_INPUT, null,                       Param::PARAM_TYPE_STR));

      $procedure->execute();
    } catch (\Exception $exception) {
      throw new \Exception("Erro ao inserir região Imposto Itemcol: " . $exception->getMessage(), 500);
    }
  }

  public function updatePaymentTerm(array $data, array $supplier = null, array $product = null, array $paymentSupplier): void {
    try {
      $query = "UPDATE ITEMCOLE
                SET CDFORMPGTO = :P_CDFORMPGTO,
                  IDTPFOPGTCOL = :P_IDTPFOPGTCOL,
                  NRDIAPGFXCOL = :P_NRDIAPGFXCOL,
                  NRPARCPGTCOL = :P_NRPARCPGTCOL,
                  IDUTDIAPGCOL = :P_IDUTDIAPGCOL
                WHERE CDFILIAL = :P_CDFILIAL
                  AND NRCOLEPRECO = :P_NRCOLEPRECO
                  AND CDPRODUTO = :P_CDPRODUTO
                  AND CDMARCA = LPAD(:P_CDMARCA, 5, '0')
                  AND CDFORNECED = :P_CDFORNECED
                  AND CDREGIAO = :P_CDREGIAO";

      $params = [
        "P_CDFILIAL"     => $data["CDFILIAL"],
        "P_NRCOLEPRECO"  => $data["NRCOLEPRECO"],
        "P_CDPRODUTO"    => $product["CDPRODUTO"] ?? $data["CDPRODUTO"],
        "P_CDMARCA"      => $product["CDMARCA"] ?? $data["CDMARCA"],
        "P_CDFORNECED"   => $supplier["CDFORNECED"] ?? $data["CDFORNECED"],
        "P_CDREGIAO"     => $data["CDREGIAO"],
        "P_CDFORMPGTO"   => $paymentSupplier[0]["CDFORMPGTO"],
        "P_IDTPFOPGTCOL" => $paymentSupplier[0]["IDTPFORPGTFO"],
        "P_NRDIAPGFXCOL" => $paymentSupplier[0]["NRDIAPGFIXFO"],
        "P_NRPARCPGTCOL" => $paymentSupplier[0]["NRPARCPGTOFO"],
        "P_IDUTDIAPGCOL" => $paymentSupplier[0]["IDUTDIAPGTFO"]
      ];

      $this->util->crud($query, $params);
    } catch (\Exception $exception) {
      throw new \Exception("Erro ao prazo de pagamento: " . $exception->getMessage(), 500);
    }
  }

  public function deleteSupplierRegister(array $data, array $supplier = null, array $product = null): void {
    try {
      $query = "DELETE FROM PZOPGTITMCOL
                WHERE CDFILIAL = :P_CDFILIAL
                  AND NRCOLEPRECO = :P_NRCOLEPRECO
                  AND CDFORNECED = :P_CDFORNECED
                  AND CDPRODUTO = :P_CDPRODUTO
                  AND CDMARCA  = LPAD(:P_CDMARCA, 5, '0')
                  AND CDREGIAO = :P_CDREGIAO
                  AND NRORG = :P_NRORG";

      $params = [
        "P_CDFILIAL"    => $data["CDFILIAL"],
        "P_NRCOLEPRECO" => $data["NRCOLEPRECO"],
        "P_CDPRODUTO"   => $product["CDPRODUTO"] ?? $data["CDPRODUTO"],
        "P_CDMARCA"     => $product["CDMARCA"] ?? $data["CDMARCA"],
        "P_CDFORNECED"  => $supplier["CDFORNECED"] ?? $data["CDFORNECED"],
        "P_CDREGIAO"    => $data["CDREGIAO"],
        'P_NRORG'       => $data["NRORG"]
      ];

      $this->util->crud($query, $params);
    } catch (\Exception $exception) {
      throw new \Exception("Erro ao deletar registro fornecedor: " . $exception->getMessage(), 500);
    }
  }

  public function insertSupplier(array $data, array $supplier = null, array $product = null, array $installment): array {
    try {
      $query = "INSERT INTO PZOPGTITMCOL
                  (
                    CDFILIAL,
                    NRCOLEPRECO,
                    CDFORNECED,
                    CDPRODUTO,
                    CDMARCA,
                    CDREGIAO,
                    NRDIASPGTCOL,
                    VRPERCPGTCOL,
                    NRORG
                  )
                    SELECT :P_CDFILIAL, :P_NRCOLEPRECO, :P_CDFORNECED, :P_CDPRODUTO,
                      LPAD(:P_CDMARCA, 5, '0'), :P_CDREGIAO,
                      :P_NRDIASPGTCOL, :P_VRPERCPGTCOL, :P_NRORG
                    FROM DUAL
                    WHERE NOT EXISTS (SELECT 1
                      FROM PZOPGTITMCOL P
                      WHERE P.CDFILIAL   = :P_CDFILIAL
                        AND P.NRCOLEPRECO  = :P_NRCOLEPRECO
                        AND P.CDFORNECED   = :P_CDFORNECED
                        AND P.CDPRODUTO    = :P_CDPRODUTO
                        AND P.CDMARCA      = LPAD(:P_CDMARCA, 5, '0')
                        AND P.CDREGIAO     = :P_CDREGIAO
                        AND P.NRDIASPGTCOL = :P_NRDIASPGTCOL )";

      $params = [
        "P_CDFILIAL"     => $data["CDFILIAL"],
        "P_NRCOLEPRECO"  => $data["NRCOLEPRECO"],
        "P_CDPRODUTO"    => $product["CDPRODUTO"] ?? $data["CDPRODUTO"],
        "P_CDMARCA"      => $product["CDMARCA"] ?? $data["CDMARCA"],
        "P_CDFORNECED"   => $supplier["CDFORNECED"] ?? $data["CDFORNCED"],
        "P_CDREGIAO"     => $data["CDREGIAO"],
        "P_NRDIASPGTCOL" => $data[0]["NRDIASPGTOFO"],
        "P_VRPERCPGTCOL" => $data[0]["VRPERCPGTOFO"],
        'P_NRORG'        => $data["NRORG"]
      ];

      return $this->util->crud($query, $params);
    } catch (\Exception $exception) {
      throw new \Exception("Erro ao inserir fornecedor: " . $exception->getMessage(), 500);
    }
  }

  public function forcTrigger(array $data, array $supplier = null, array $product = null): void {
    try {
      $query = "UPDATE ITEMCOLE
                SET IDTPFOPGTCOL = IDTPFOPGTCOL
                WHERE CDFILIAL = :P_CDFILIAL
                  AND NRCOLEPRECO = :P_NRCOLEPRECO
                  AND CDPRODUTO = :P_CDPRODUTO
                  AND CDMARCA = LPAD(:P_CDMARCA, 5, '0')
                  AND CDFORNECED = :P_CDFORNECED
                  AND CDREGIAO = :P_CDREGIAO";

      $params = [
        "P_CDFILIAL"    => $data["CDFILIAL"],
        "P_NRCOLEPRECO" => $data["NRCOLEPRECO"],
        "P_CDPRODUTO"   => $product["CDPRODUTO"] ?? $data["CDPRODUTO"],
        "P_CDMARCA"     => $product["CDMARCA"] ?? $data["CDMARCA"],
        "P_CDFORNECED"  => $supplier["CDFORNECED"] ?? $data["CDFORNECED"],
        "P_CDREGIAO"    => $data["CDREGIAO"]
      ];
      $this->util->crud($query, $params);
    } catch (\Exception $exception) {
      throw new \Exception("Erro forcTrigger: " . $exception->getMessage(), 500);
    }
  }

  public function insertSupplierProduct(array $data, array $supplier = null, array $product = null): array {
    try {
      $query = "INSERT INTO PZOPGTITMCOL
                  (
                    CDFILIAL,
                    NRCOLEPRECO,
                    CDFORNECED,
                    CDPRODUTO,
                    CDMARCA,
                    CDREGIAO,
                    NRDIASPGTCOL,
                    VRPERCPGTCOL,
                    NRORG
                  )
                SELECT ITEMCOLE.CDFILIAL, ITEMCOLE.NRCOLEPRECO,
                  ITEMCOLE.CDFORNECED, ITEMCOLE.CDPRODUTO,
                  ITEMCOLE.CDMARCA, ITEMCOLE.CDREGIAO,
                  PZOPGTPRFO.NRDIASPGPRFO, PZOPGTPRFO.VRPERCPGPRFO,
                  PZOPGTPRFO.NRORG
                FROM ITEMCOLE, PZOPGTPRFO
                WHERE ITEMCOLE.IDTPFOPGTCOL IS NOT NULL
                  AND ITEMCOLE.CDFILIAL = :P_CDFILIAL
                  AND ITEMCOLE.NRCOLEPRECO = :P_NRCOLEPRECO
                  AND ITEMCOLE.CDFORNECED = :P_CDFORNECED
                  AND ITEMCOLE.CDPRODUTO = :P_CDPRODUTO
                  AND ITEMCOLE.CDMARCA = LPAD(:P_CDMARCA, 5, '0')
                  AND ITEMCOLE.CDREGIAO = :P_CDREGIAO
                  AND ITEMCOLE.NRORG = PZOPGTPRFO.NRORG
                  AND PZOPGTPRFO.NRORG = :P_NRORG
                  AND PZOPGTPRFO.CDPRODUTO = :P_PRODFORN
                  AND PZOPGTPRFO.CDFORNECED = ITEMCOLE.CDFORNECED";

      $params = [
        "P_CDFILIAL"    => $data["CDFILIAL"],
        "P_NRCOLEPRECO" => $data["NRCOLEPRECO"],
        "P_CDPRODUTO"   => $data["CDPRODUTO"],
        "P_CDMARCA"     => $product["CDMARCA"] ?? $data["CDMARCA"],
        "P_CDFORNECED"  => $product["CDFORNECED"] ?? $data["CDFORNECED"],
        "P_CDREGIAO"    => $data["CDREGIAO"],
        "P_PRODFORN"    => $data["CDREGIAO"],
        'P_NRORG'       => $data["NRORG"]
      ];

      return $this->util->crud($query, $params);
    } catch (\Exception $exception) {
      throw new \Exception("Erro ao inserir produto fornecedor: " . $exception->getMessage(), 500);
    }
  }
}