<?php

namespace SupNext\Services\QuoteSheetGrid;

use Doctrine\DBAL\Connection;
use SupNext\Util\Util;

use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;

class DeleteRegionService {
  private $connection;
  private $util;

  public function __construct(Connection $connection, Util $util) {
    $this->connection = $connection;
    $this->util       = $util;
  }

  public function deleteRegion(array $data): void {
    try {
      $query = "DELETE FROM ITEMCOLE
                WHERE CDFILIAL = :P_CDFILIAL
                      AND NRCOLEPRECO = :P_NRCOLEPRECO
                      AND CDREGIAO = :P_CDREGIAO
                      AND CDFORNECED = NVL(:P_CDFORNECED, CDFORNECED)
                      AND NRORG = :P_NRORG";

      $params = [
        "P_CDFILIAL"    => $data["CDFILIAL"],
        "P_NRCOLEPRECO" => $data["NRCOLEPRECO"],
        "P_CDREGIAO"    => $data["CDREGIAO"],
        "P_CDFORNECED"  => $data["CDFORNECED"] ?? null,
        'P_NRORG'       => $data["NRORG"]
      ];

      $this->util->crud($query, $params);
    } catch (Exception $exception) {
      throw new \Exception("Erro ao executar query para deletar região: " . $exception->getMessage(), 500);
    }
  }

  public function procItemcoleAtuCustosup(array $data): void {
    try {
      $procedure = new StoredProcedure($this->connection, "ITEMCOLE_ATU_CUSTOSUP");

      $procedure->addParam(new Param("P_CDFILIAL",     Param::PARAM_INPUT, $data["CDFILIAL"],     Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_NRCOLEPRECO",  Param::PARAM_INPUT, $data["NRCOLEPRECO"],  Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_CDMARCA",      Param::PARAM_INPUT, $data["CDMARCA"],      Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_CDREGIAO",     Param::PARAM_INPUT, $data["CDREGIAO"],     Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_CDPRODUTO",    Param::PARAM_INPUT, $data["CDPRODUTO"],    Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_CDFORNECED",   Param::PARAM_INPUT, $data["CDFORNECED"],   Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param("P_NRSEQPRELIST", Param::PARAM_INPUT, $data["NRSEQPRELIST"], Param::PARAM_TYPE_STR));

      $procedure->execute();
    } catch (\Exception $exception) {
      throw new \Exception("Erro ao executar procedure para deletar região: " . $exception->getMessage(), 500);
    }
  }
}