<?php

namespace SupNext\Services\QuoteSheetGrid;

use Doctrine\DBAL\Connection;
use SupNext\Util\Util;

use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;

class ImportCollectionOrderService {
    private $connection;
    private $util;

    public function __construct(Connection $connection, Util $util) {
        $this->connection = $connection;
        $this->util = $util;
    }

    public function acmInsertColeta($row, $cdfilial, $nrColeta, $idutistatcot) {
        try {
            $query = "INSERT INTO COLETAPR
                                  (
                                        CDFILIAL,
                                        NRCOLEPRECO,
                                        DTCOLEPREC,
                                        CDOPERADOR,
                                        NRSEMAINIC,
                                        NRSEMAFINAL,
                                        DTINIVALCOL,
                                        DTFINVALCOL,
                                        IDTIPCOLREG,
                                        CDREGIAO,
                                        IDSITUCOLETA
                                    )
                                    VALUES
                                    (
                                        :CDFILIAL,
                                        :NRCOLEPRECO,
                                        :DTCOLEPREC,
                                        :CDOPERADOR,
                                        :NRSEMAINIC,
                                        :NRSEMAFINAL,
                                        :DTINIVALCOL,
                                        :DTFINVALCOL,
                                        :IDTIPCOLREG,
                                        :CDREGIAO,
                                        :IDSITUCOLETA
                                    )";
            $data = date('d/m/Y');
            $idsitucoleta = $idutistatcot == 'S' ? 'A' : 'C';

            //campo OBS_FORNECECOR mantido para considerar arquivos antigos.
            $obsFornecedor = isset($row['OBS_FORNECECOR']) ? $row['OBS_FORNECECOR'] : null;
            $obsFornecedor = isset($row['OBS_FORNECEDOR']) ? $row['OBS_FORNECEDOR'] : null;

            $param = [
                "CDFILIAL"     => $cdfilial,
                "NRCOLEPRECO"  => $nrColeta,
                "DTCOLEPREC"   => $data,
                "CDOPERADOR"   => $row['COMPRADOR'],
                "NRSEMAINIC"   => $this->toDateWW($row["VALIDADE_INI"]),
                "NRSEMAFINAL"  => $this->toDateWW($row["VALIDADE_FIN"]),
                "DTINIVALCOL"  => $row['VALIDADE_INI'],
                "DTFINVALCOL"  => $row['VALIDADE_FIN'],
                'IDTIPCOLREG'  => 'E',
                'CDREGIAO'     => $row['REGIAO'],
                'IDSITUCOLETA' => $idsitucoleta
            ];

            return $this->util->crud($query, $param);
        } catch (\Exception $e) {
            throw new \Exception('QUOTATION_ERROR');
        }
    }

    private function toDateWW($date) {
        $dateTime = new DateTime($date);
        $dateWW = $dateTime->format('W');

        return $dateWW;
    }

    public function acmInsertItemCole($row, $cdproduto, $cdfilial, $nrColeta, $idutistatcot, $fornecedor, $cdsAuxiliar, $nrorg) {
        try {
            $query = "INSERT INTO ITEMCOLE
                                  (
                                        CDFILIAL,
                                        NRCOLEPRECO,
                                        CDFORNECED,
                                        CDREGIAO,
                                        CDPRODUTO,
                                        CDMARCA,
                                        DTINIVALITM,
                                        DTFINVALITM,
                                        VRPRECUNIITM,
                                        IDTIPFREITM,
                                        CDFORMPGTO,
                                        IDTPFOPGTCOL,
                                        NRPARCPGTCOL,
                                        NRDIAPGFXCOL,
                                        IDUTDIAPGCOL,
                                        QTACMPITMCOL,
                                        VRPERDESITM,
                                        DSOBSITMCOL
                                  )
                                  VALUES
                                  (
                                        :P_CDFILIAL,
                                        :P_NRCOLEPRECO,
                                        :P_CDFORNECED,
                                        :P_CDREGIAO,
                                        :P_CDPRODUTO,
                                        LPAD(:P_CDMARCA, 5, '0'),
                                        :P_DTINIVALITM,
                                        :P_DTFINVALITM,
                                        :P_VRPRECUNIITM,
                                        :P_IDTIPFREITM,
                                        :P_CDFORMPGTO,
                                        :P_IDTPFOPGTCOL,
                                        :P_NRPARCPGTCOL,
                                        :P_NRDIAPGFXCOL,
                                        :P_IDUTDIAPGCOL,
                                        :P_QTACMPITMCOL,
                                        :P_VRPERDESITM,
                                        :P_DSOBSITMCOL
                                  )";

            $obsFornecedor = isset($row['OBS_FORNECEDOR'])
                ? $row['OBS_FORNECEDOR']
                : (isset($row['OBS_FORNECECOR'])
                    ? $row['OBS_FORNECECOR'] : null);

            $param = [
                "P_CDFILIAL"     => $cdfilial,
                "P_NRCOLEPRECO"  => $nrColeta,
                "P_CDFORNECED"   => $fornecedor,
                "P_CDREGIAO"     => $row['REGIAO'],
                "P_CDPRODUTO"    => $cdproduto,
                "P_CDMARCA"      => $row['MARCA'],
                "P_DTINIVALITM"  => $row['VALIDADE_INI'],
                "P_DTFINVALITM"  => $row['VALIDADE_FIN'],
                'P_VRPRECUNIITM' => $row['VRUNI'],
                'P_IDTIPFREITM'  => $cdsAuxiliar["IDTPFRETFORN"],
                'P_CDFORMPGTO'   => $cdsAuxiliar["CDFORMPGTO"],
                'P_IDTPFOPGTCOL' => $cdsAuxiliar["IDTPFORPGTFO"],
                'P_NRPARCPGTCOL' => $cdsAuxiliar["NRDIAPGFIXFO"],
                'P_NRDIAPGFXCOL' => $cdsAuxiliar["NRPARCPGTOFO"],
                'P_IDUTDIAPGCOL' => $cdsAuxiliar["IDUTDIAPGTFO"],
                'P_QTACMPITMCOL' => $row['QTDE'],
                'P_VRPERDESITM'  => 0,
                "P_DSOBSITMCOL"  => $obsFornecedor
            ];

            $this->util->crud($query, $param);

            if ($cdsAuxiliar["IDTPFORPGTFO"] == "V") {
                $rowsPagamento = $this->pagamentoV($fornecedor, $nrorg);
                $this->insertPzopgtitmcol($rowsPagamento, $row, $cdfilial, $nrColeta, $fornecedor, $cdproduto);
            }
        } catch (\Exception $e) {
            throw new \Exception('QUOTATION_ERROR');
        }
    }

    public function pagamentoV($supplier, $nrorg) {
       $payment = $this->util->filteredDataSet(["CDFORNECED" => $supplier, "NRORG" => $nrorg], "paymentV");
       
       return $payment;
    }

    public function insertPzopgtitmcol($rowsPagamento, $row, $cdfilial, $nrColeta, $fornecedor, $cdproduto) {
        try {
            foreach ($rowsPagamento as &$rowPagamento) {
                $query = "INSERT INTO PZOPGTITMCOL
                                      (
                                            CDFILIAL,
                                            NRCOLEPRECO,
                                            CDFORNECED,
                                            CDPRODUTO,
                                            CDMARCA,
                                            NRDIASPGTCOL,
                                            VRPERCPGTCOL,
                                            CDREGIAO
                                        )
                                        VALUES
                                        (
                                            :P_CDFILIAL,
                                            :P_NRCOLEPRECO,
                                            :P_CDFORNECED,
                                            :P_CDPRODUTO,
                                            LPAD(:P_CDMARCA, 5, '0'),
                                            :P_NRDIASPGTCOL,
                                            :P_VRPERCPGTCOL,
                                            :P_CDREGIAO
                                        )";

                $param = [
                    "P_CDFILIAL"     => $cdfilial,
                    "P_NRCOLEPRECO"  => $nrColeta,
                    "P_CDFORNECED"   => $fornecedor,
                    "P_CDPRODUTO"    => $cdproduto,
                    "P_CDMARCA"      => $row["MARCA"],
                    "P_NRDIASPGTCOL" => $rowPagamento["NRDIASPGTOFO"],
                    "P_VRPERCPGTCOL" => $rowPagamento["VRPERCPGTOFO"],
                    "P_CDREGIAO"     => $row["REGIAO"]
                ];

                return $this->util->crud($query, $param);
            }
        } catch (\Exception $e) {
            throw new \Exception('QUOTATION_ERROR');
        }
    }

    public function procImposto($cdempresa, $cdincesta, $row, $cdfilial, $nrColeta, $fornecedor, $cdproduto) {
        try {
            $procedure = new StoredProcedure($this->connection, 'IMPOSTO_ITEMCOL');

            $procedure->addParam(new Param('P_EMPRESA',    Param::PARAM_INPUT, $cdempresa,          Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_INSCESTA',   Param::PARAM_INPUT, $cdincesta,          Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_FORNECEDOR', Param::PARAM_INPUT, $fornecedor,         Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_FILCENT',    Param::PARAM_INPUT, $cdfilial,           Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_LANCCOLP',   Param::PARAM_INPUT, $nrColeta,           Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDFORNECED', Param::PARAM_INPUT, $fornecedor,         Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDREGIAO',   Param::PARAM_INPUT, $row["REGIAO"],      Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDPRODUTO',  Param::PARAM_INPUT, $cdproduto,          Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDMARCA',    Param::PARAM_INPUT, $row["MARCA"],       Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_INIVALID',   Param::PARAM_INPUT, $row["VALIDADE_INI"], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_TRANSACAO',  Param::PARAM_INPUT, null,                Param::PARAM_TYPE_STR));

            return $procedure->execute();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function acmForcTrigger($cdfilial, $nrColeta, $row, $fornecedor, $cdproduto) {
        try {
            $query = "UPDATE ITEMCOLE
                      SET IDTPFOPGTCOL = IDTPFOPGTCOL
                      WHERE CDFILIAL = :P_FILIAL
                            AND NRCOLEPRECO = :P_NRCOLEPRECO
                            AND CDREGIAO = :P_CDREGIAO
                            AND CDFORNECED  = :P_FORNECEDOR
                            AND CDPRODUTO = :P_CDPRODUTO
                            AND CDMARCA = LPAD(:P_CDMARCA, 5, '0')";
            $params = [
                "P_FILIAL"      => $cdfilial,
                "P_NRCOLEPRECO" => $nrColeta,
                "P_CDREGIAO"    => $row['REGIAO'],
                "P_FORNECEDOR"  => $fornecedor,
                "P_CDPRODUTO"   => $cdproduto,
                "P_CDMARCA"     => $row['MARCA']
            ];

            $this->util->crud($query, $params);
        } catch (\Exception $e) {
            throw new \Exception('RECORD_ERROR');
        }
    }
}
