import { ModalService, LoadingService, FileInput } from '@zeedhi/common';
import { Metadata, Http, I18n } from '@zeedhi/core';
import { AppState } from '@/controllers/AppState';
import { Utilities } from '@/util/Utilities';

export class ImportCollectionOrderController {
  private utilities: Utilities;

  private appState: AppState;

  constructor() {
    this.utilities = new Utilities();
    this.appState = AppState.getInstance();
  }

  public async createModal(): Promise<void> {
    await ModalService.createFromJson(
      '/modals/quoteSheetGrid/importCollectionOrderModal',
      true,
    );
  }

  public destroyModal(): void {
    ModalService.getModal('importCollectionOrderModal')?.destroy();
  }

  public openModal(): void {
    ModalService.getModal('importCollectionOrderModal').show();
  }

  public closeModal(): void {
    ModalService.getModal('importCollectionOrderModal').hide();
  }

  public async importCollectionOrder(): Promise<void> {
    const file = Metadata.getInstance<FileInput>('file');

    LoadingService.show();

    try {
      const base64 = await this.utilities.convertXlsxToBase64(file) as string;
      const params = {
        FILE: {
          name: file.value[0].name,
          content: base64.substring(base64.indexOf(',') + 1),
        },
        CDFILIAL: this.appState.CDFILIAL,
        NRORG: this.appState.NRORG,
      };

      const { data } = await Http.post('/importCollectionOrder', params);

      if (data.error) {
        const errorsSorted = data.error.sort((a, b) => a.ROWNUM - b.ROWNUM);

        this.utilities.showErrorDialog(errorsSorted.map(
          (erro) => `Linha ${erro.ROWNUM}${erro.COLUMN ? ` e Coluna ${erro.COLUMN}` : ''}: ${I18n.translate(erro.ERROR_CODE,
            {
              product_code: erro.VALUE,
              brand_code: erro.VALUE,
              supplier_code: erro.VALUE,
            })
          }`,
        ).join('</br>'));
      }
    } catch (error) {
      this.utilities.showErrorDialog('sup::ERROR');
    } finally {
      LoadingService.hide();
    }
  }
}
