import { TekGrid } from '@zeedhi/teknisa-components-common';
import { ModalService, Text } from '@zeedhi/common';
import { Metadata } from '@zeedhi/core';
import { AppState } from '@/controllers/AppState';

export class ProductsInformationController {
  private appState: AppState;

  constructor() {
    this.appState = AppState.getInstance();
  }

  public async createModal(): Promise<void> {
    await ModalService.createFromJson(
      '/modals/quoteSheetGrid/productInformationModal',
      true,
    );
  }

  public destroyModal(): void {
    ModalService.getModal('productInformationModal')?.destroy();
  }

  public openModal(): void {
    ModalService.getModal('productInformationModal').show();
    setTimeout(() => {
      this.setValues();
    }, 200);
  }

  public closeModal(): void {
    ModalService.getModal('productInformationModal').hide();
  }

  private setValues(): void {
    const grid = Metadata.getInstance<TekGrid>('multipleLevelsProductsGrid');
    const { currentRow } = grid.datasource;
    const VRPERDESITM = currentRow.VRPERDESITM > 0
      ? currentRow.VRPRECUNIITM - (currentRow.VRPERDESITM * currentRow.VRPRECUNIITM) / 100
      : parseFloat(currentRow.VRPRECUNIITM).toFixed(5);
    const fieldsMap = {
      price_eco: `<strong>Pr.Eco.:</strong> ${parseFloat(currentRow.VRPRECECOITM ?? 0).toFixed(5)}`,
      price_eco_PI: `<strong>Pr.Eco. PI:</strong> ${parseFloat(currentRow.VRPRECOITMPI ?? 0).toFixed(5)}`,
      quantity: `<strong>Quantidade:</strong> ${parseFloat(currentRow.QTACMPITMCOL ?? 0).toFixed(3)}`,
      discount: `<strong>Desconto:</strong> ${parseFloat(currentRow.VRPERDESITM ?? 0).toFixed(2)}%`,
      unit_price: `<strong>Preço unitário:</strong> ${parseFloat(currentRow.VRINDICEITMCOL ?? 0).toFixed(5)}`,
      unit_price2: `<strong>Preço unitário:</strong> ${VRPERDESITM}`,
      quantity_available: `<strong>Quantidade disponível:</strong> ${currentRow.QTDISPPRODMARC ?? ''}`,
    };
    const fieldsList = [
      'price_eco',
      'price_eco_PI',
      'quantity',
      'discount',
      'unit_price',
      'unit_price2',
      'quantity_available',
    ];

    Metadata.getInstance<Text>('unit_price2').isVisible = this.appState.registeredParams.IDUTINDICECOL === 'S';

    fieldsList.forEach((field) => {
      Metadata.getInstance<Text>(field).text = fieldsMap[field];
    });
  }
}
