import { TekGrid } from '@zeedhi/teknisa-components-common';
import { ModalService, TextInput } from '@zeedhi/common';
import { Metadata } from '@zeedhi/core';
import { Utilities } from '@/util/Utilities';

export class QuotationLogController {
  private utilities!: Utilities;

  constructor() {
    this.utilities = new Utilities();
  }

  public async createModal(): Promise<void> {
    await ModalService.createFromJson(
      '/modals/quoteSheetGrid/quotationLogModal',
      true,
    );
  }

  public destroyModal(): void {
    ModalService.getModal('quotationLogModal')?.destroy();
  }

  public openModal(): void {
    const quoteGrid = Metadata.getInstance<TekGrid>('multipleLevelsQuotationGrid');

    if (!quoteGrid.selectedRows.length || quoteGrid.selectedRows.length > 1) {
      this.utilities.showWarningDialog('Selecione apenas uma cotação para acessar Log de Cotação');
      return;
    }

    ModalService.getModal('quotationLogModal').show();

    setTimeout(() => {
      const quoteInput = Metadata.getInstance<TextInput>('quotation_log_quotation');
      quoteInput.value = quoteGrid.selectedRows[0].NRCOLEPRECO;
    }, 200);
  }

  public closeModal(): void {
    ModalService.getModal('quotationLogModal').hide();
  }
}
